# -*- coding: utf-8 -*-
"""
Functions to remove publications/links from nodes/edges list, if they can no longer be reached

"""

__authors__ = "Donna Löding, Alina Molkentin, Xinyi Tang, Judith Große, Malte Schokolowski"
__email__ = "cis-project2021@zbh.uni-hamburg.de"
__status__ = "Production"
#__copyright__ = ""
#__credits__ = ["", "", "", ""]
#__license__ = ""
#__version__ = ""
#__maintainer__ = ""

import sys  
from pathlib import Path
sys.path.append("../")

from input.interface import InputInterface as Input
from Kanten_Vergleich import back_to_valid_edges


def delete_ref_nodes_rec(pub):
    '''
    :param pub: pub go get deleted after recursive call
    :type pub:  Publication
    
    function that removes nodes of group "height", if they aren't reachable from input nodes
    '''
    for reference in pub.references:
        for ref_pub in processed_list:
            if (ref_pub.doi_url == reference):
                
                # to find a cyclus and avoid recursion error
                if (reference not in pub.citations):
                    delete_ref_nodes_rec(ref_pub)

    # removes publication from list after recursion and if it's not of group input
    if (pub.group != "input"):
        processed_list.remove(pub)


def delete_cit_nodes_rec(pub):
    '''
    :param pub: publication to be removed after recursive call
    :type pub:  Publication
    
    function that removes publications of group "depth" from input_list, if they aren't reachable from input nodes
    '''
    for citation in pub.citations:
        for cit_pub in processed_list:
            if (cit_pub.doi_url == citation):

                # to find a cyclus and avoid recursion error
                if (citation not in pub.references):
                    delete_cit_nodes_rec(cit_pub)

    # removes publication from list after recursion and if it's not of group input               
    if (pub.group != "input"):
        processed_list.remove(pub)


def delete_nodes_and_edges(input_list, deleted_nodes, old_doi_input_list, old_edges_list):
    '''
    :param input_list:      list of publications to get reduced
    :type input_list:       List[Publication]

    :param deleted_nodes:       list of input dois which are not in new call
    :type deleted_nodes:        List[String]

    :param old_doi_input_list:  list of input dois from old call
    :type old_doi_input_list:   List[String]

    :param old_edges_list:      list of links between publications from old call
    :type old_edges_list:       List[List[String,String]]
    
    function to start recursive node removal for references and citations and to return edge list to valid state
    '''

    # global list to save the process of removing unneeded publications
    global processed_list
    processed_list = input_list.copy()

    for del_node in deleted_nodes:
            for pub in processed_list:
                if (del_node == pub.doi_url):

                    # checks for every reference if it is citet more than once. If not it calls deletion function
                    for reference in pub.references:
                        only_reference = True
                        for ref_cit in processed_list:
                            if (reference == ref_cit.doi_url):
                                for citation in ref_cit.citations:
                                    if ((citation in old_doi_input_list) and (citation != del_node)):
                                        only_reference = False
                                        break
                            if (only_reference == False):
                                break
                        if (only_reference):
                            delete_ref_nodes_rec(pub)

                    # checks for every citation if it cites more than once. If not it calls deletion function
                    for citation in pub.citations:
                        only_reference = True
                        for cit_ref in processed_list:
                            if (citation == cit_ref.doi_url):
                                for reference in cit_ref.references:
                                    if ((reference in old_doi_input_list) and (reference != del_node)):
                                        only_reference = False
                                        break
                            if (only_reference == False):
                                break
                        if (only_reference):
                            delete_cit_nodes_rec(pub)

                    # removes publication of type input after start of recursive call to both directions
                    processed_list.remove(pub)

    valid_edges = back_to_valid_edges(old_edges_list, processed_list)
    return(processed_list, valid_edges)